// Copyright (C) 2009 Martin Sandve Alnes
//
// This file is part of SyFi.
//
// SyFi is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// SyFi is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SyFi. If not, see <http://www.gnu.org/licenses/>.
//
// First added:  2009-01-01
// Last changed: 2009-04-01
//
// This demo program solves Poissons equation in 2D with
// the solution projected to a fixed value in the middle.

#include <vector>
#include <dolfin.h>
#include "generated_code/Forms.h"

using std::vector;
using namespace dolfin;
using namespace Forms;

class Source: public Expression 
{
public:
  void eval(Array<double>& values, const Array<double>& x) const
  {
    double dx = x[0] - 0.5;
    double dy = x[1] - 0.5;
    values[0] = 0.0*dx*dy;
  }
};

class Value: public Expression 
{
public:
  void eval(Array<double>& values, const Array<double>& x) const
  {
    double dx = x[0] - 0.5;
    double dy = x[1] - 0.5;
    values[0] = 2.0*exp(-10.0*(dx*dx+dy*dy))-1;
  }
};

class BoundaryValue: public Expression 
{
public:
  void eval(Array<double>& values, const Array<double>& x) const
  {
    double dx = x[0] - 0.5;
    double dy = x[1] - 0.5;
    values[0] = dx + dy;
  }
};

class ValueDomain: public SubDomain
{
  bool inside(const Array<double>& x, bool on_boundary) const
  {
    double dx = x[0] - 0.5;
    double dy = x[1] - 0.5;
    double r = 0.2;
    return (dx*dx + dy*dy) < r*r;
  }
};

class DirichletBoundary: public SubDomain
{
  bool inside(const Array<double>& x, bool on_boundary) const
  {
    return x[1] < DOLFIN_EPS || x[1] > 1.0-DOLFIN_EPS;
  }
};

int main()
{
    unsigned n = 100;
    UnitSquare mesh(n, n);

    //MeshFunction<unsigned> domains(mesh, 2);
    boost::shared_ptr< MeshFunction<unsigned> > domains(new MeshFunction<unsigned>(mesh, 2));
    ValueDomain vdomain;
    vdomain.mark(*domains, 1);
    
    BilinearForm::TrialSpace V(mesh);
    CoefficientSpace_f F(mesh);
    CoefficientSpace_g G(mesh);
    
    BilinearForm a(V, V);
    LinearForm L(V);

    L.set_cell_domains(domains);
    a.set_cell_domains(domains);

    Source f;
    Value g;
    BoundaryValue ubc;
    
    L.f = f;
    L.g = g;
    
    DirichletBoundary boundary;
    DirichletBC bc(V, ubc, boundary);
    
    VariationalProblem problem(a, L, bc);
    Function u(V);
    problem.solve(u);
    
    File ufile("u.pvd");
    ufile << u;
    
    //plot(u);
}
